// Description: 
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#include "FreeOTFEHashImpl.h"
#include "FreeOTFEHashMD.h"

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"


// Include TomLibCrypt libraries...
#include <mycrypt_custom.h>


// =========================================================================
// Hash driver init function
// devExt - The device extension to be initialized
NTSTATUS
ImpHashDriverExtDetailsInit(
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;
    int idx = -1;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsInit\n"));


    // -- POPULATE DRIVER IDENTIFICATION --
    RtlZeroMemory(devExt->DriverTitle, sizeof(devExt->DriverTitle));
    RtlCopyMemory(
                  devExt->DriverTitle,
                  DRIVER_TITLE,
                  strlen(DRIVER_TITLE)
                 );

    devExt->DriverGUID = DRIVER_GUID;
    devExt->DriverVersionID = DRIVER_HASH_VERSION;


    // -- POPULATE HASHES SUPPORTED --
    devExt->HashDetails = ExAllocatePool(
                                         NonPagedPool,
                                         (sizeof(HASH) * HASHES_SUPPORTED)
                                        );    

    devExt->HashCount = HASHES_SUPPORTED;



    // -- MD2 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_MD2,
                  strlen(DRIVER_HASH_TITLE_MD2)
                 );

    devExt->HashDetails[idx].Length = (md2_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (md2_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_MD2;


    // -- MD4 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_MD4,
                  strlen(DRIVER_HASH_TITLE_MD4)
                 );

    devExt->HashDetails[idx].Length = (md4_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (md4_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_MD4;


    // -- MD5 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_MD5,
                  strlen(DRIVER_HASH_TITLE_MD5)
                 );

    devExt->HashDetails[idx].Length = (md5_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (md5_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_MD5;



    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsInit\n"));

    return status;
}


// =========================================================================
// Hash driver cleardown function
// devExt - The device extension to be cleared down
NTSTATUS
ImpHashDriverExtDetailsCleardown(    
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsCleardown\n"));

    if (devExt->HashDetails != NULL)
        {
        ExFreePool(devExt->HashDetails);
        }

    devExt->HashDetails = NULL;
    devExt->HashCount = 0;

    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsCleardown\n"));

    return status;
}


// =========================================================================
// Hash function
NTSTATUS
ImpHashHashData(
    IN      GUID* HashGUID,
    IN      unsigned int DataLength,  // In bits
    IN      unsigned char* Data,
    IN OUT  unsigned int* HashLength,  // In bits
    OUT     unsigned char* Hash
)
{
    NTSTATUS status = STATUS_SUCCESS;
    hash_state md;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashHashData\n"));
    if (IsEqualGUID(&HASH_GUID_MD5, HashGUID))
        {
        if (*HashLength < (md5_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (md5_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            md5_desc.init(&md);
            md5_desc.process(&md, Data, (DataLength / 8));
            md5_desc.done(&md, Hash);
        
            *HashLength = (md5_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_MD4, HashGUID))
        {
        if (*HashLength < (md4_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (md4_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            md4_desc.init(&md);
            md4_desc.process(&md, Data, (DataLength / 8));
            md4_desc.done(&md, Hash);
        
            *HashLength = (md4_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_MD2, HashGUID))
        {
        if (*HashLength < (md2_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (md2_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            md2_desc.init(&md);
            md2_desc.process(&md, Data, (DataLength / 8));
            md2_desc.done(&md, Hash);
        
            *HashLength = (md2_desc.hashsize * 8);
            }
            
        }
    else
        {
        DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("driver doesn't recognise GUID\n"));
        status = STATUS_INVALID_PARAMETER;
        }
        
    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashHashData\n"));
      
    return status;
}


// =========================================================================
// =========================================================================

